package org.apache.turbine.om.security;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java classes
import java.util.Map;
import java.util.HashMap;
import java.util.Collections;

// Turbine utility classes
import org.apache.turbine.om.BaseObject;

// Turbine Security Classes
import org.apache.turbine.om.security.*;
import org.apache.turbine.util.security.*;

/**
 * This class represents a generic object used in the Aaccess Control Lists. 
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @author <a href="mailto:Rafal.Krzewski@e-point.pl">Rafal Krzewski</a>
 * @version $Id: SecurityObject.java,v 1.2 2000/11/19 02:33:48 rafal Exp $
 */
public class SecurityObject extends BaseObject implements Comparable
{
    /** The name of this object. */
    private String name;

    /** The attributes of this object. */
    private Map attributes;

    /**
     * Constructs a new SecurityObject
     */
    public SecurityObject()
    {
        this("");
    }

    /** 
     * Constructs a new SecurityObject with the sepcified name.
     *
     * @param name The name of the new object.
     */
    public SecurityObject( String name )
    {
        this.name = name;
        this.attributes = Collections.synchronizedMap(new HashMap());
    }

    /**
     * Returns the name of this object.
     *
     * @return The name of the object.
     */
    public String getName()
    {
        return name;
    }

    /**
     * Sets the name of this object.
     *
     * @param name The name of the object.
     */
    public void setName(String name)
    {
        this.name = name;
    }

    /**
     * Returns a Map containg this object's attributes.
     *
     * @return the object's attributes.
     */
    public Map getAttributes()
    {
        return attributes;
    }

    /**
     * Replaces this object's attributes with the specifed Map.
     *
     * @param attributes The new attributes of the object.
     */
    public void setAttributes( Map attributes )
    {
        this.attributes = attributes;
    }

    /**
     * Retrieves the value of specific attribute of this object.
     *
     * @param name the name of the attribute
     * @return the value of the attribute
     */
    public Object getAttribute( String name ) 
    {
        return attributes.get(name);
    }

    /**
     * Sets the value of specific attribute of this object.
     *
     * @param name the name of the attribute
     * @return the value of the attribute
     */
    public void setAttribute( String name, Object value )
    {
        attributes.put(name, value);
    }

    /**
     * Used for ordering SecurityObjects.
     *
     * @param obj The Object to compare to.
     * @return -1 if the name of the other object is lexically greater than this
     *         group, 1 if it is lexically lesser, 0 if they are equal.
     */
    public int compareTo(Object obj)
    {
        if(this.getClass() != obj.getClass())
            throw new ClassCastException();
        String name1 = ((SecurityObject)obj).getName();
        String name2 = this.getName();

        return name2.compareTo(name1);
    }

    /**
     * Returns a textual representation of this object, consisted by 
     * it's name and attributes.
     *
     * @return  a textual representation of this group.
     */
    public String toString()
    {
        StringBuffer sb = new StringBuffer();
        sb.append(name);
        sb.append(":");
        sb.append(attributes.toString());
        return sb.toString();
    }
}
