package org.apache.turbine.services.assemblerbroker;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// JDK Stuff
import java.util.*;

// Turbine Stuff
//import org.apache.turbine.services.Service;
import org.apache.turbine.services.*;
import org.apache.turbine.services.resources.TurbineResources;
import org.apache.turbine.util.*;
import org.apache.turbine.modules.*;
import org.apache.turbine.util.assemblerbroker.*;

/**
 * TurbineAssemblerBrokerService allows assemblers (like screens,
 * actions and layouts) to be loaded from one or more AssemblerFactory
 * classes.  AssemblerFactory classes is registered with this broker
 * by adding them to the TurbineResources.properties file.
 *
 *
 * @author <a href="mailto:leon@opticode.co.za">Leon Messerschmidt</a>
 */
public class TurbineAssemblerBrokerService
    extends TurbineBaseService
    implements AssemblerBrokerService

{
    /** A structure that holds the registered AssemblerFactories*/
    private Hashtable factories = null;

    /**
     * Get a list of AssemblerFactories of a certain type
     */
    private Vector getFactoryGroup (String type)
    {
        if (!factories.containsKey (type))
        {
            factories.put (type, new Vector());
        }

        return (Vector)factories.get(type);

    }

    /**
     * Utiltiy method to register all factories for a given
     * type.
     */
    private void registerFactories (String type)
    {
        String key = TurbineServices.SERVICE_PREFIX+
                     AssemblerBrokerService.SERVICE_NAME+
                     "."+
                     type;

        String[] names = TurbineResources.getStringArray(key);

        Log.note ("Registering "+names.length+" "+type+"s");


        for (int i=0; i<names.length; i++)
        {
            try
            {
                Object o = Class.forName ( names[i] ).newInstance();
                registerFactory ( type, (AssemblerFactory)o );
            }
            catch (Exception e)
            {
                Log.error ("Assembler Factory Load Error:");
                Log.error (e.toString());
            }

        }

    }


    /**
     * Initializes the AssemblerBroker and loads the AssemblerFactory
     * classes registerd in TurbineResources.Properties.
     */
    public void init()
    {
        if (getInit()) return;

        org.apache.turbine.util.Log.note ( "Assembler Broker init()....starting!");

        factories = new Hashtable();

        registerFactories (AssemblerBrokerService.ACTION_TYPE);
        registerFactories (AssemblerBrokerService.SCREEN_TYPE);
        registerFactories (AssemblerBrokerService.NAVIGATION_TYPE);
        registerFactories (AssemblerBrokerService.LAYOUT_TYPE);
        registerFactories (AssemblerBrokerService.PAGE_TYPE);

        setInit(true);

        org.apache.turbine.util.Log.note ( "Assembler Broker init()....finished!");
    }

    /**
     * Register a new AssemblerFactory under a certain type
     */
    public void registerFactory( String type, AssemblerFactory factory )
    {
        getFactoryGroup(type).add (factory);
    }

    /**
     * Attempt to retrieve an Assembler of a given type with
     * a name.  Cycle through all the registered AssemblerFactory
     * classes of type and retrun the first non-null assembly
     * found.  If an assembly was not found return null.
     */
    public Assembler getAssembler( String type, String name ) throws Exception
    {
        Vector facs = getFactoryGroup(type);

        for (int i=0; i<facs.size(); i++)
        {
            AssemblerFactory fac = (AssemblerFactory)facs.get(i);
            Assembler assembler = null;
            try
            {
                assembler = fac.getAssembler (name);
            }
            catch (Exception e)
            {
                Log.error ("TAB "+e);
                throw e;
            }


            if (assembler != null) return assembler;
        }

        return null;
    }

}
