package org.apache.turbine.services.jsp;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Jyve",
 *    "Apache Jyve", "Jyve Project", "Apache Jyve Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Stuff 
import java.util.*;
import java.io.*;

// Java Servlet Classes
import javax.servlet.*;
import javax.servlet.http.*;

// Turbine Stuff
import org.apache.turbine.services.*;
import org.apache.turbine.services.resources.*;
import org.apache.turbine.util.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.services.jsp.util.*;

/*
 * This is a Service that can process FreeMarker templates 
 * from within a Turbine Screen.
 * Here's an example of how you might use it from a screen:<br>
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 */
public class TurbineJspService extends TurbineBaseService
    implements JspService
{
    /** The base path prepended to filenames given in arguments */
    private String path;    

    /** The buffer size for the output stream. */
    private int bufferSize;

    /** Default constructor. */
    public TurbineJspService()
        throws Exception
    {
    }
    
    /**
     * Called during Turbine.init()
     */
    public void init( ServletConfig config )
    {
        try
        {
            org.apache.turbine.util.Log.note ( "TurbineJspService early init()....starting!");
            initJsp(config);
            setInit(true);
            org.apache.turbine.util.Log.note ( "TurbineJspService early init()....finished!");
        }
        catch (Exception e)
        {
            org.apache.turbine.util.Log.error ( "Cannot initialize TurbineJspService!" );
            org.apache.turbine.util.Log.error (e);
        }
   } 
    
    /**
     * Adds some convenience objects to the request.  For example an instance
     * of JspLink which can be used to generate links to other templates.
     * 
     * @param RunData the turbine rundata object 
     */
    public void addDefaultObjects(RunData data)
    {
        HttpServletRequest req = data.getRequest();
        req.setAttribute(LINK, new JspLink(data));
        req.setAttribute(RUNDATA, data);
    }

    /**
     * The buffer size
     * 
     */
    public int getDefaultBufferSize()
    {
        return bufferSize;
    }


    /**
     * Process the request 
     * 
     * @param RunData 
     * @param String the filename of the template.
     */
    public void handleRequest(RunData data, String filename) 
    {
        handleRequest(data, filename, false);
    }
    
    /**
     * Process the request 
     * 
     * @param RunData 
     * @param String the filename of the template.
     * @param boolean whether to perform a forward or include.
     */
    public void handleRequest(RunData data, String filename, boolean isForward) 
    {                                
        // get the RequestDispatcher for the JSP
        RequestDispatcher dispatcher = data.getServletContext()
            .getRequestDispatcher(path + filename);
        try
        {
            if (isForward)
            {
                // forward the request to the JSP
                dispatcher.forward( data.getRequest(), data.getResponse() ); 
            }
            else
            {
                data.getOut().flush();
                // include the JSP
                dispatcher.include( data.getRequest(), data.getResponse() ); 
            }
        }
        catch(Exception e)
        {
            org.apache.turbine.util.Log.error ("Error encountered processing a template:" + filename);
            org.apache.turbine.util.Log.error (e);
            try
            {
                data.getOut().print("Error encountered processing a template: " 
                    + e.toString());
            }
            catch(IOException ioe) {}    
        }
    }
        
 
   
    /**
     * This method sets up the template cache.
     */
    private void initJsp(ServletConfig config)
        throws Exception
    {
        ServletContext context = config.getServletContext();
        Properties props = getProperties();

        path = props.getProperty("templates", "");
        /* not working properly - removing until I can investigate (jdm)
        if (path.startsWith("/"))
        {
            path = path.substring(1);
        }
        if (path.length() > 0 && !path.endsWith("/"))
        {
            path += "/";
        }
        */
        bufferSize = Integer.parseInt(props.getProperty("buffer.size", "2000000"));
    }

}








