package org.apache.turbine.torque.ant;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

import java.util.Date;

// Velocity Stuff
import org.apache.velocity.Context;
import org.apache.velocity.texen.ant.TexenTask;

// Turbine Stuff
import org.apache.turbine.torque.model.AppData;
import org.apache.turbine.torque.transform.XmlToAppData;

import org.apache.turbine.torque.util.TorqueStringUtil;

/**
 * An ant task for generating output by using Velocity
 *
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @version $Id: VTorqueTask.java,v 1.4 2000/11/29 06:36:55 mcnally Exp $
 */

public class VTorqueTask extends TexenTask
{
    private AppData app;

    private String xmlFile;
    private String targetPackage;
    private String targetDatabase;
    private String addSaveMethod;
    
    /**
     * Get the current xml file
     */
    public String getXmlFile ()
    {
        return xmlFile;
    }

    /**
     * Set the xml file
     */
    public void setXmlFile(String v)
    {
        xmlFile = v;
    }

    /**
     * Get the current target package
     */
    public String getTargetPackage ()
    {
        return targetPackage;
    }

    /**
     * Set the current target package.  This is where
     * generated java classes will live.
     */
    public void setTargetPackage (String v)
    {
        targetPackage = v;
    }

    /**
     * Get the current target package
     */
    public String getTargetDatabase ()
    {
        return targetDatabase;
    }

    /**
     * Set the current target package.  This is where
     * generated java classes will live.
     */
    public void setTargetDatabase (String v)
    {
        targetDatabase = v;
    }
    /**
     * Get whether to add code to support save method
     */
    public String getAddSaveMethod ()
    {
        return addSaveMethod;
    }

    /**
     * Set whether to add code to support save method
     */
    public void setAddSaveMethod (String v)
    {
        addSaveMethod = v;
    }

    public Context initControlContext()
    {
        Context context = new Context();
//        app = new AppData();
        XmlToAppData xmlParser = new XmlToAppData();
        app = xmlParser.parseFile(xmlFile);
        
        context.put("databaseModel", app);
        context.put("targetDatabase", targetDatabase);
        context.put("package", targetPackage);
        context.put("tstrings", new TorqueStringUtil());
        context.put("now", new Date().toString());
        context.put("addSaveMethod", new Boolean(addSaveMethod));
        return context;
    }
}



