package org.apache.turbine.torque.model;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// JDK
import java.io.*;
import java.util.*;

// Xerces
import org.xml.sax.*;

/**
 * A class for holding application data structures.
 *
 * @author <a href="mailto:leon@opticode.co.za>Leon Messerschmidt</a>
 * @version $Id: AppData.java,v 1.1 2000/11/07 23:38:34 jvanzyl Exp $
 */
public class AppData
{

    private Vector tableVector = new Vector();
    private Column curColumn;

    /**
     * Default Constructor
     */
    public AppData()
    {
    }

    /**
     * Return an array of all tables
     */
    public Table[] getTables()
    {
        int size = tableVector.size();
        Table[] tbls = new Table[size];
        for (int i = 0; i < size; i++)
        {
            tbls[i] = (Table)tableVector.elementAt(i);
        }
        return tbls;
    }


    /**
     * Return the table with the specified name.
     * @return A Table object.  If it does not exist it returns null
     */
    public Table getTable (String name)
    {
        for (Enumeration e = tableVector.elements() ; e.hasMoreElements() ;)
        {
            Table tbl = (Table ) e.nextElement();
            if (tbl.getName().equals(name))
            {
                return tbl;
            }
        }
        return null;
    }

    /**
     * An utility method to add a new table from
     * an xml attribute.
     */
    public Table addTable(Attributes attrib)
    {
        Table tbl = new Table();
        tbl.loadFromXML (attrib);
        addTable (tbl);
        return tbl;
    }

    /**
     * Add a table to the vector and sets the
     * AppData property to this AppData
     */
    public void addTable(Table tbl)
    {
        tbl.setAppData (this);
        tableVector.addElement(tbl);
    }


    /**
     * Creats a string representation of this AppData.
     * The representation is given in xml format.
     */
    public String toString()
    {
        StringBuffer result = new StringBuffer();

        result.append ("<database>\n");
        for (Enumeration e = tableVector.elements() ; e.hasMoreElements() ;)
        {
            result.append (e.nextElement());
        }
        result.append ("</database>");
        return result.toString();
  }
}
