package org.apache.turbine.torque.model;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// JDK
import java.io.*;
import java.util.*;

// Xerces
import org.xml.sax.*;
import org.xml.sax.helpers.*;

import org.apache.turbine.torque.util.TorqueStringUtil;

/**
 * A Class for holding data about a column used in an Application.
 *
 * @author <a href="mailto:leon@opticode.co.za">Leon Messerschmidt</a>
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @version $Id: Column.java,v 1.2 2000/11/10 00:07:04 jvanzyl Exp $
 */
public class Column
{
    private String name;
    private String javaName = null;
    private boolean isNotNull;
    private int size;
    private String type;
    private Object columnType;
    private Table parentTable;
    private boolean isPrimaryKey;
    private boolean isUnique;
    private boolean isAutoIncrement;
    private String defaultValue;
    
    /**
     * Default Constructor
     */
    public Column()
    {
    }


    /**
     * Creates a new column and set the name
     */
    public Column(String name)
    {
        this.name = name;
    }

    /**
     * Imports a column from an XML specification
     */
    public void loadFromXML (Attributes attrib)
    {

        //Name
        name = attrib.getValue("name");

        javaName = attrib.getValue("javaName");

        //Primary Key
        String primaryKey = attrib.getValue("primaryKey");
        if ("true".equals (primaryKey))  //Do it this way to avoid NullPointerException
        {
                isPrimaryKey = true;
        }
        else
        {
                isPrimaryKey = false;
        }

        /*
         * If this column is a primary key then it
         * can't be null.
         */
        if ("true".equals (primaryKey))  //Do it this way to avoid NullPointerException
        {
                isNotNull = true;
        }

        String notNull = attrib.getValue("required");
        if (notNull != null && "true".equals(notNull))
            isNotNull = true;
        else
            isNotNull = false;

        /*
         * AutoIncrement/Sequences
         */
        String autoIncrement = attrib.getValue("autoIncrement");
        if (autoIncrement != null && "true".equals(autoIncrement))
                isAutoIncrement = true;
        else
                isAutoIncrement = false;

        /*
         * Default column value.
         */
        defaultValue = attrib.getValue("default");

        //size
        String s = attrib.getValue("size");
        //size = attrib.getValue("size");
        
        if (s != null)
        {
            size = Integer.parseInt (s);
        }
        else
        {
            size = 0;
        }

        //type
        //String t = attrib.getValue("type");
        type = attrib.getValue("type");
        
        if (type.equals("INTEGER"))
        {
            columnType = new Integer(0);
        }

        else if (type.equals("VARCHAR"))
        {
            columnType = "";
        }
        else if (type.equals("TIMESTAMP"))
        {
            columnType = new java.util.Date();
        }
        else
        {
            columnType = "";
        }
    }


    /**
     * Returns table.column
     */
    public String getFullyQualifiedName()
    {
        return parentTable.getName()+"."+name;
    }

    /**
     * Get the name of the column
     */
    public String getName()
    {
        return name;
    }

    /**
     * Set the name of the column
     */
    public void setName(String newName)
    {
        name = newName;
    }


    /**
     * Get name to use in Java sources
     */

    public String getJavaName()
    {
        if (javaName == null)
        {
            return TorqueStringUtil.removeUnderScores(name);
        }
        else
        {
            return javaName;
        }
    }

    /**
     * Set name to use in Java sources
     */

    public void setJavaName(String javaName)
    {
        this.javaName = javaName;
    }


    /**
     * Set the parent Table of the column
     */
    public void setTable(Table parent)
    {
        parentTable = parent;
    }

    /**
     * Get the parent Table of the column
     */
    public Table getTable()
    {
        return parentTable;
    }

    /**
     * Returns the Name of the table the column is in
     */
    public String getTableName()
    {
        return parentTable.getName();
    }

    /**
     * Return the isNotNull property of the column
     */
    public boolean isNotNull()
    {
        return isNotNull;
    }

    /**
     * Set the isNotNull property of the column
     */
    public void setNotNull(boolean status)
    {
        isNotNull = status;
    }

    /**
     * Set if the column is a primary key or not
     */
    public void setPrimaryKey(boolean pk)
    {
        isPrimaryKey = pk;
    }

    /**
     * Return true if the column is a primary key
     */
    public boolean isPrimaryKey()
    {
        return isPrimaryKey;
    }

    /**
     * Set true if the column is UNIQUE
     */
    public void setUnique (boolean u)
    {
        isUnique = u;
    }

    /**
     * Get the UNIQUE property
     */
    public boolean isUnique()
    {
        return isUnique;
    }

    /**
     * Utility method to determine if this column
     * is a foreign key.
     */
    public boolean isForeignKey()
    {
        return (parentTable.getForeignKey (this.name) != null);
    }


    /**
     * Utility method to get the related table of this
     * column if it is a foreign key or part of a foreign
     * key
     */
    public String getRelatedTableName()
    {
        ForeignKey fk = parentTable.getForeignKey (this.name);
        if (fk == null) return null;
        return fk.getForeignTableName();
    }


    /**
     * Utility method to get the related column of this
     * local column if this column is a foreign key or
     * part of a foreign key.
     */
    public String getRelatedColumnName()
    {
        ForeignKey fk = parentTable.getForeignKey (this.name);
        if (fk == null) return null;
        return fk.getLocalForeignMapping().get(this.name).toString();
    }

    /**
     * Returns the colunm type
     */
    public void setType(String type)
    {
        this.type = type;
    }


    /**
     * Set the column type from a string property
     * (normally a string from an sql input file)
     */
    public void setTypeFromString (String typeName, String size)
    {
        String tn = typeName.toUpperCase();
        if (tn.indexOf ("CHAR") != -1)
        {
            columnType = "";
            try
            {
                this.size = Integer.parseInt (size);
            }
            catch (Exception e)
            {
                this.size = 0;
            }

        }
        else if (tn.indexOf ("INT") != -1)
        {
            columnType = new Integer (0);
        }
        else  if (tn.indexOf ("DATE") != -1 || tn.indexOf ("TIME") != -1)
        {
            columnType = new java.util.Date();
        }
        else
        {
            columnType = "";
            try
            {
                this.size = Integer.parseInt (size);
            }
            catch (Exception e)
            {
                this.size = 0;
            }
        }
    }

    /**
     * Returns the colunm type
     */
    public String getType()
    {
        return type;
    }


    /**
     * Utility method to see if the column is a string
     */
    public boolean isString()
    {
        return (type instanceof String);
    }
    
    /**
     * String representation of the column. This
     * is an xml representation.
     */
    public String toString()
    {
        StringBuffer result = new StringBuffer();
        result.append(" <column name=\""+name+"\"");

        if (javaName != null)
        {
            result.append(" javaName=\""+javaName+"\"");
        }


        if (isPrimaryKey)
        {
            result.append(" primaryKey=\""+isPrimaryKey+"\"");
        }

        if (isNotNull)
        {
            result.append(" required=\"true\"");
        }
        else
        {
            result.append(" required=\"false\"");
        }

        result.append(" type=\"").append (type).append("\"");
        
        if (size != 0)
            result.append(" size=\""+size+"\"");

        result.append(">");
        result.append("</column>\n");
        return result.toString();
    }

    /**
     * Returns the size of the column
     */
    public int getSize()
    {
        return size;
    }

    /**
     * Set the size of the column
     */
    public void setSize(int newSize)
    {
        size = newSize;
    }


    /**
     * Return the size in brackets for use in an sql
     * schema if the type is String.  Otherwise return
     * an empty string
     */
    public String printSize()
    {
        if (size != 0)
        {
            return "("+size+")";
        }

        return "";
    }

    /**
     * Return a string that will give this column
     * a default value.
     */
     public String getDefaultSetting()
     {
        if (defaultValue == null)
            return "";
        else
            return "default " + defaultValue;
     }

    /**
     * Return auto increment/sequence string for
     * the target database. We need to pass in the
     * props for the target database!
     */
     public boolean isAutoIncrement()
     {
        return isAutoIncrement;
     }
}
