package org.apache.turbine.torque.model;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// JDK
import java.io.*;
import java.util.*;

// Xerces
import org.xml.sax.*;
import org.xml.sax.helpers.*;

import org.apache.turbine.torque.util.TorqueStringUtil;

/**
 * A Class for holding data about a table used in an Application.
 *
 * @author <a href="mailto:leon@opticode.co.za">Leon Messerschmidt</a>
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @version $Id: Table.java,v 1.3 2000/12/02 02:07:21 mcnally Exp $
 */
public class Table
{

    //private AttributeListImpl attributes;
    private Vector columnVector;
    private Vector foreignKeys;
    private Vector indices;
    private Vector unices;
    private Vector idMethodParameters;
    private String name;
    private String javaName;
    private String idMethod;
    private AppData tableParent;

    /**
     * Default Constructor
     */
    public Table()
    {
        this (null);
    }

    /**
     * Constructs a table object with a name
     */
    public Table(String name)
    {
        this.name = name;
        columnVector = new Vector();
        foreignKeys = new Vector();
        indices = new Vector();
        unices = new Vector();
    }

    /**
     * Load the table object from an xml tag.
     */
    public void loadFromXML (Attributes attrib)
    {
        //attributes = new AttributeListImpl(attrib);
        //name
        name = attrib.getValue("name");
        javaName = attrib.getValue("javaName");
        idMethod = attrib.getValue("idMethod");
    }


    /**
     * A utility function to create a new column
     * from attrib and add it to this table.
     */
    public Column addColumn(Attributes attrib)
    {
        Column col = new Column();
        col.loadFromXML (attrib);
        addColumn (col);

        return col;
    }

    /**
     * Adds a new column to the column vector and set the
     * parent table of the column to the current table
     */
    public void addColumn(Column col)
    {
        col.setTable (this);
        columnVector.addElement(col);
    }

    /**
     * A utility function to create a new foreign key
     * from attrib and add it to this table.
     */
    public ForeignKey addForeignKey(Attributes attrib)
    {
        ForeignKey fk = new ForeignKey();
        fk.loadFromXML (attrib);
        addForeignKey (fk);

        return fk;
    }

    /**
     * Adds a new FK to the FK vector and set the
     * parent table of the column to the current table
     */
    public void addForeignKey(ForeignKey fk)
    {
        fk.setTable (this);
        foreignKeys.addElement(fk);
    }

    /**
    /**
     * A utility function to create a new id method parameter
     * from attrib and add it to this table.
     */
    public IdMethodParameter addIdMethodParameter(Attributes attrib)
    {
        IdMethodParameter imp = new IdMethodParameter();
        imp.loadFromXML (attrib);
        addIdMethodParameter (imp);

        return imp;
    }

    /**
     * Adds a new id method parameter to the vector and set the
     * parent table of the column to the current table
     */
    public void addIdMethodParameter(IdMethodParameter imp)
    {
        imp.setTable (this);
        if (idMethodParameters == null)
        {
            idMethodParameters = new Vector(10);
        }
        idMethodParameters.addElement(imp);
    }

    /**
     * Adds a new index to the index vector and set the
     * parent table of the column to the current table
     */
    public void addIndex(Index index)
    {
        index.setTable (this);
        indices.addElement(index);
    }

    /**
     * A utility function to create a new index
     * from attrib and add it to this table.
     */
    public Index addIndex(Attributes attrib)
    {
        Index index = new Index();
        index.loadFromXML (attrib);
        addIndex (index);

        return index;
    }

    /**
     * Adds a new Unique to the Unique vector and set the
     * parent table of the column to the current table
     */
    public void addUnique(Unique unique)
    {
        unique.setTable (this);
        unices.addElement(unique);
    }

    /**
     * A utility function to create a new Unique
     * from attrib and add it to this table.
     */
    public Unique addUnique(Attributes attrib)
    {
        Unique unique = new Unique();
        addUnique (unique);

        return unique;
    }


    /**
     * Get the name of the Table
     */
    public String getName()
    {
        return name;
    }

    /**
     * Set the name of the Table
     */
    public void setName(String newName)
    {
        name = newName;
    }

    /**
     * Get name to use in Java sources
     */
    public String getJavaName()
    {
        if (javaName == null)
        {
            //TO DO: Do some pritty printing here
            return TorqueStringUtil.removeUnderScores(name);
        }
        else
        {
            return javaName;
        }
    }

    /**
     * Set name to use in Java sources
     */
    public void setJavaName(String javaName)
    {
        this.javaName = javaName;
    }

    /**
     * Get the method for generating pk's
     */
    public String getIdMethod()
    {
        if (idMethod == null)
        {
            return "idbroker";
        }
        else
        {
            return idMethod;
        }
    }

    /**
     * Set the method for generating pk's
     */
    public void setIdMethod(String idMethod)
    {
        this.idMethod = idMethod;
    }

    /**
     * Returns an Array containing all the columns in the table
     */
    public Column[] getColumns()
    {
        int size = columnVector.size();
        Column[] tbls = new Column[size];
        for (int i = 0; i < size; i++)
        {
            tbls[i] = (Column)columnVector.elementAt(i);
        }
        return tbls;
    }

    /**
     * Utility method to get the number of columns in this table
     */
    public int getNumColumns()
    {
        return columnVector.size();
    }

    /**
     * Returns an Array containing all the FKs in the table
     */
    public ForeignKey[] getForeignKeys()
    {
        int size = foreignKeys.size();
        ForeignKey[] tbls = new ForeignKey[size];
        for (int i = 0; i < size; i++)
        {
            tbls[i] = (ForeignKey)foreignKeys.elementAt(i);
        }
        return tbls;
    }

    /**
     * Returns an Array containing all the FKs in the table
     * !FIXME!
     */
    public Vector getIdMethodParameters()
    {
        return idMethodParameters;
    }
    

    /**
     * Returns an Array containing all the FKs in the table
     */
    public Index[] getIndices()
    {
        int size = indices.size();
        Index[] tbls = new Index[size];
        for (int i = 0; i < size; i++)
        {
            tbls[i] = (Index)indices.elementAt(i);
        }
        return tbls;
    }

    /**
     * Returns an Array containing all the FKs in the table
     */
    public Unique[] getUnices()
    {
        int size = unices.size();
        Unique[] tbls = new Unique[size];
        for (int i = 0; i < size; i++)
        {
            tbls[i] = (Unique)unices.elementAt(i);
        }
        return tbls;
    }


    /**
     * Returns a Spesified column.
     * @return Return a Column object or null if it does not exist.
     */
    public Column getColumn(String name)
    {
        String curName;

        for (Enumeration e = columnVector.elements() ; e.hasMoreElements() ;)
        {
            Column col = (Column) e.nextElement();
            curName = col.getName();
            if (curName.equals(name))
            {
                return col;
            }
        }
        return null;
    }

    /**
     * Return the first foreign key that includes col in it's list
     * of local columns.  Eg. Foreign key (a,b,c) refrences tbl(x,y,z)
     * will be returned of col is either a,b or c.
     * @return Return a Column object or null if it does not exist.
     */
    public ForeignKey getForeignKey (String col)
    {

        for (Enumeration e = foreignKeys.elements() ; e.hasMoreElements() ;)
        {
            ForeignKey key = (ForeignKey)e.nextElement();
            if (key.getLocalColumns().contains (col))
                return key;
        }

        return null;
    }


    /**
     * Returns true if the table contains a spesified column
     */
    public boolean containsColumn(Column col)
    {
        return columnVector.contains (col);
    }

    /**
     * Returns true if the table contains a spesified column
     */
    public boolean containsColumn(String name)
    {
        return (getColumn (name) != null);
    }

    /**
     * Set the parent of the table
     */
    public void setAppData(AppData parent)
    {
        tableParent = parent;
    }

    /**
     * Get the parent of the table
     */
    public AppData getAppData()
    {
        return tableParent;
    }

    /**
     * Creates a string representation of this table. This
     * is an xml representation.
     */
    public String toString()
    {
        StringBuffer result = new StringBuffer();

        result.append ("<table name=\"").append(name);

        if (javaName != null)
        {
            result.append(" javaname=\""+javaName+"\"");
        }

        if (idMethod != null)
        {
            result.append(" idMethod=\""+idMethod+"\"");
        }

        result.append("\">\n");

        for (Enumeration e = columnVector.elements() ; e.hasMoreElements() ;)
        {
            result.append (e.nextElement());
        }
        for (Enumeration e = foreignKeys.elements() ; e.hasMoreElements() ;)
        {
            result.append (e.nextElement());
        }
        for (Enumeration e = idMethodParameters.elements() ; 
             e.hasMoreElements() ;)
        {
            result.append (e.nextElement());
        }

        result.append ("</table>\n");

        return result.toString();
    }

    /**
     * Print the primary key(s) with commas between them.
     */
    public String printPrimaryKeys()
    {
        StringBuffer result = new StringBuffer();
        boolean comma=false;

        Enumeration enum = columnVector.elements();

        while (enum.hasMoreElements())
        {
            Column col = (Column)enum.nextElement();
            if (col.isPrimaryKey())
            {
                if (comma) result.append (','); else comma=true;
                result.append (col.getName());
            }
        }

        return result.toString();
    }

}
