package org.apache.turbine.torque.sql;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

import java.io.*;
import java.util.Vector;


/**
 * A simple Scanner implementation that scans an
 * sql file into usable tokens.  Used by SQLToAppData.
 *
 * @author <a href="mailto:leon@opticode.co.za>Leon Messerschmidt</a>
 * @version $Id: SQLScanner.java,v 1.1 2000/11/07 23:39:00 jvanzyl Exp $
 */
public class SQLScanner
{
    static private final String white = "\f\r\t\n ";
    static private final String alfa = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
    static private final String numer = "0123456789";
    static private final String alfanum = alfa+numer;
    static private final String special = ";(),";

    private Reader in;
    private int chr;
    private String token;
    private Vector tokens;

    private int line;
    private int col;

    /**
     * Creates a new scanner with no Reader
     */
    public SQLScanner()
    {
        this (null);
    }

    /**
     * Creates a new scanner with an Input Reader
     */
    public SQLScanner (Reader input)
    {
        setInput (input);
    }

    /**
     * Set the Input
     */
    public void setInput (Reader input)
    {
        in = input;
    }


    /**
     * Reads the next character and increments the line and
     * column counters.
     */
    private void readChar() throws IOException
    {
        chr = in.read();
        if ((char)chr == '\n' || (char)chr == '\r' || (char)chr == '\f')
        {
            col=0;
            line++;
        } else col++;
    }

    /**
     * Scans an identifier.
     */
    private void scanIdentifier () throws IOException
    {
        token = "";
        char c = (char)chr;
        while (chr != -1 && white.indexOf(c) < 1 && special.indexOf(c) < 1)
        {
            token = token+(char)chr;
            readChar();
            c = (char)chr;
        }
        tokens.addElement(new Token (token,line,col));
    }

    /**
     * Scan the input Reader and returns a list
     * of tokens.
     */
    public Vector scan () throws IOException
    {
        line = 1;
        col = 0;
        tokens = new Vector();
        readChar();
        while (chr != -1)
        {
            char c = (char)chr;

            if (alfanum.indexOf(c) >= 0)
            {
                scanIdentifier();
            }
            else if (special.indexOf(c) >= 0)
            {
                tokens.addElement(new Token (""+c,line,col));
                readChar();
            }
            else chr = in.read();
        }
        return tokens;
    }

}
