package org.apache.turbine.torque.transform;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// JDK classes
import java.io.*;
import java.util.*;

// Xerces classes
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.apache.xerces.parsers.*;

import org.apache.turbine.torque.model.*;

/**
 * The is an utility classes that is used to parse an input 
 * xml schema file and creates and AppData java structure.
 * It uses apache Xerces to do the xml parsing.
 *
 * @author <a href="mailto:leon@opticode.co.za">Leon Messerschmidt</a>
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @version $Id: XmlToAppData.java,v 1.2 2000/11/29 06:36:56 mcnally Exp $
 */
public class XmlToAppData extends DefaultHandler
{
    private AppData app;
    private Table currTable;
    private ForeignKey currFK;
    private Index currIndex;
    private Unique currUnique;

    /**
     * Default custructor
     */
    public XmlToAppData()
    {
        app = null;
    }


    /**
     * Parse and xml input file and returns a newly
     * created and populated AppData structure
     */
    public AppData parseFile(String xmlFile)
    {
        try
        {
            app = new AppData();

            SAXParser parser = new SAXParser();
            
            // We don't use an external content handler - we use this object
            parser.setContentHandler(this);
            
            // Validate the input file
            parser.setFeature("http://apache.org/xml/features/validation/dynamic",true);
            
            FileReader fr = new FileReader (xmlFile);
            BufferedReader br = new BufferedReader (fr);
            try
            {
                InputSource is = new InputSource (br);
                parser.parse(is);
            }
            finally
            {
                br.close();
            }
        }
        catch (Exception e)
        {
            //System.out.println("Error : "+e);
            e.printStackTrace();
        }
        return app;
    }

    /**
     * Handles opening elements of the xml file.
     */
    public void startElement(String uri, String localName, String rawName, Attributes attributes)
    {
        if (rawName.equals("table"))
        {
            currTable = app.addTable(attributes);
        }
        else if (rawName.equals("column"))
        {
            currTable.addColumn(attributes);
        }
        else if (rawName.equals("foreign-key"))
        {
            currFK = currTable.addForeignKey(attributes);
        }
        else if (rawName.equals("reference"))
        {
            currFK.addReference(attributes);
        }
        else if (rawName.equals("index"))
        {
            currIndex = currTable.addIndex(attributes);
        }
        else if (rawName.equals("index-column"))
        {
            currIndex.addColumn(attributes);
        }
        else if (rawName.equals("unique"))
        {
            currUnique = currTable.addUnique(attributes);
        }
        else if (rawName.equals("unique-column"))
        {
            currUnique.addColumn(attributes);
        }
        else if (rawName.equals("id-method-parameter"))
        {
            currTable.addIdMethodParameter(attributes);
        }
    }
}
