package org.apache.turbine.torque.util;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

import java.io.*;
import java.util.*;

import org.apache.velocity.texen.util.StringUtil;
import org.apache.turbine.util.db.map.*;

/**
 * A general string and settings utililty that is added to the context
 * as $strings.
 *
 * @author <a href="mailto:leon@opticode.co.za>Leon Messerschmidt</a>
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @version $Id: TorqueStringUtil.java,v 1.2 2000/11/08 00:47:23 jonbolt Exp $
 */

public class TorqueStringUtil extends StringUtil
{
    /**
     * Return a string reperesention of the ColumnMap type
     */
    static public String getType (Object foo)
    {
        String type = "string";
        if (foo instanceof String)
        {
            type = "string";
        }
        else if (foo instanceof Integer)
        {
            type = "integer";
        }
        else if (foo instanceof Date)
        {
            type = "date";
        }
        return type;
    }

    /**
     * returns the type of object foo. fi it doesn't know what it is,
     * then it makes it "string"
     */
    static public String getVillageType (String foo)
    {
        String type = "asString()";
        
        if (foo.equals("VARCHAR"))
        {
          type = "asString()";
        }
        else if (foo.equals("INTEGER"))
        {
          type = "asInt()";
        }
        else if (foo.equals("TIMESTAMP"))
        {
          type = "asDate()";
        }
        
        return type;
    }

    /**
     * Get the Java Type
     */
    static public String getJavaType (String foo)
    {
        String type = "String";
        
        if (foo.equals("VARCHAR"))
        {
          type= "String";
        }
        else if (foo.equals("INTEGER"))
        {
          type = "int";
        }
        else if (foo.equals("TIMESTAMP"))
        {
          type = "Date";
        }
        return type;
    }

    /**
     * Return the SQL type of the Object depending
     * on the database you're using.  If your database
     * is not defined a set of default values will be
     * used.
     */
    static public String getSQLType (Object foo, String db) throws IOException, FileNotFoundException
    {
        try
        {
            Properties props = new Properties ();
            try
            {
                props.load ( new FileInputStream (db) );
            }
            catch (FileNotFoundException e1)
            {
                try
                {
                    props.load ( new FileInputStream (getDefault (db)) );
                }
                catch (FileNotFoundException e2)
                {
                    throw e1;
                }

            }

            return props.getProperty (getType (foo));
        }
        catch (Exception e)
        {
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter (sw);
            e.printStackTrace (pw);
            pw.close();
            return sw.toString();
        }

    }

    /**
     * Creates a default name from a file name that
     * does not exist.  For example /mystuff/notexist.txt
     * becomes /mystuff/default.txt
     */
    static public String getDefault (String input)
    {
        String newInput=null;

        int pos = input.lastIndexOf ('/');
        if (pos == -1)
        {
            newInput = "defaut";
        }
        else
        {
            newInput = input.substring (0,pos+1)+"default";
        }

        pos = input.lastIndexOf ('.');
        if (pos != -1)
        {
            newInput = newInput+input.substring (pos);
        }

        return newInput;
    }
}
